@@ -1,7 +1,11 @@
-from urllib.parse import urlencode
+from base64 import b64decode
 from html import escape
+from re import sub
+from urllib.parse import urlencode
+
 import aiohttp
 from dotmap import DotMap
+from pyrogram.types import Message
 
 
 class ARQ:
@@ -12,8 +16,8 @@ class ARQ:
 
     Parameters
     ___________
-        ARQ_API (``str``):
-            Pass ARQ_API_BASE_URL as argument
+        ARQ_API (URL: str, API_KEY: str):
+            Pass ARQ_API_BASE_URL and ARQ_API_KEY as argument
 
     Methods
     -------
@@ -83,6 +87,10 @@ class ARQ:
     tmdb(query: str = "")
         Search Something on TMDB
             Returns result object which you can access with dot notation.
+
+    quotly(messages: [Message])
+        Generate stickers from telegram message.
+            Returns base64 of the image sticker.
     """
 
     def __init__(self, api_url: str, api_key: str):
@@ -90,6 +98,10 @@ class ARQ:
         self.api_key = api_key
 
     async def _fetch(self, route, params={}):
+        if params:
+            for param in params:
+                params[param] = escape(str(params[param]))
+
         async with aiohttp.ClientSession() as session:
             async with session.get(
                 f"{self.api_url}/{route}?{urlencode(params)}",
@@ -106,12 +118,33 @@ class ARQ:
             return DotMap(response)
         raise Exception(result)
 
+    async def _post(self, route, payload={}):
+        async with aiohttp.ClientSession() as session:
+            async with session.post(
+                f"{self.api_url}/{route}",
+                headers={"X-API-KEY": self.api_key},
+                params={"payload": str(payload)},
+            ) as resp:
+                if resp.status in (
+                    401,
+                    403,
+                ):
+                    raise Exception("Invalid API key")
+                response = await resp.json()
+        ok, result = response
+        if ok:
+            response["result"] = b64decode(
+                sub("data:image/png;base64", "", response["result"])
+            )
+            return DotMap(response)
+        raise Exception(result)
+
     async def deezer(self, query: str, count: int):
         """
         Returns An Object.
 
                 Parameters:
-                        query (str): Query to search
+                        query (int): Query to search
                         count (int): Number of results to return
                 Returns:
                         result object (str): Results which you can access with dot notation, Ex - results[result_number].url
@@ -119,7 +152,9 @@ class ARQ:
                         result[result_number].title | .id | .source | .duration | .thumbnail | .artist | .url
 
         """
-        return await self._fetch("deezer", {"query": escape(query), "count": escape(count)})
+        return await self._fetch(
+            "deezer", {"query": query, "count": count}
+        )
 
     async def torrent(self, query: str):
         """
@@ -132,7 +167,7 @@ class ARQ:
 
                         result[result_number].name | .uploaded | .size | .seeds | .leechs | .magnet
         """
-        return await self._fetch("torrent", {"query": escape(query)})
+        return await self._fetch("torrent", {"query": query})
 
     async def saavn(self, query: str):
         """
@@ -145,7 +180,7 @@ class ARQ:
 
                         result[result_number].song | .album | .year | .singers | .image | .duration | .media_url
         """
-        return await self._fetch("saavn", {"query": escape(query)})
+        return await self._fetch("saavn", {"query": query})
 
     async def youtube(self, query: str):
         """
@@ -159,7 +194,7 @@ class ARQ:
 
                         result[result_number].id | .thumbnails | .title | .long_desc | .channel | .duration | .views | .publish_time | .url_suffix
         """
-        return await self._fetch("youtube", {"query": escape(query)})
+        return await self._fetch("youtube", {"query": query})
 
     async def wall(self, query: str):
         """
@@ -172,7 +207,7 @@ class ARQ:
 
                         result[result_number].id | .width | .height | .file_type | .file_size | .url_image | .url_thumb | .url_page
         """
-        return await self._fetch("wall", {"query": escape(query)})
+        return await self._fetch("wall", {"query": query})
 
     async def reddit(self, query: str):
         """
@@ -185,7 +220,7 @@ class ARQ:
 
                         result.postLink | .subreddit | .title | .url | .nsfw | .spoiler | .author | .ups | .preview
         """
-        return await self._fetch("reddit", {"query": escape(query)})
+        return await self._fetch("reddit", {"query": query})
 
     async def urbandict(self, query: str):
         """
@@ -198,7 +233,7 @@ class ARQ:
 
                         result[result_number].definition | .permalink | .thumbs_up | .sound_urls | .author | .word | .defid | .example | .thumbs_down
         """
-        return await self._fetch("ud", {"query": escape(query)})
+        return await self._fetch("ud", {"query": query})
 
     async def pornhub(self, query: str = "", page: int = 1, thumbsize: str = "small"):
         """
@@ -216,7 +251,12 @@ class ARQ:
                         result[result_number].id | .title | .duration | .views | .rating | .url | .category | .thumbnails
         """
         return await self._fetch(
-            "ph", {"query": escape(query), "page": escape(page), "thumbsize": escape(thumbsize)}
+            "ph",
+            {
+                "query": query,
+                "page": page,
+                "thumbsize": thumbsize,
+            },
         )
 
     async def phdl(self, url: str):
@@ -228,7 +268,7 @@ class ARQ:
                 Returns:
                         result object (str): Result
         """
-        return await self._fetch("phdl", {"url": escape(url)})
+        return await self._fetch("phdl", {"url": url})
 
     async def luna(self, query: str):
         """
@@ -239,7 +279,7 @@ class ARQ:
                 Returns:
                         result object (str): Result
         """
-        return await self._fetch("luna", {"query": escape(query)})
+        return await self._fetch("luna", {"query": query})
 
     async def lyrics(self, query: str):
         """
@@ -252,7 +292,7 @@ class ARQ:
 
                         results.lyrics
         """
-        return await self._fetch("lyrics", {"query": escape(query)})
+        return await self._fetch("lyrics", {"query": query})
 
     async def wiki(self, query: str):
         """
@@ -265,7 +305,7 @@ class ARQ:
 
                         results.title | .answer
         """
-        return await self._fetch("wiki", {"query": escape(query)})
+        return await self._fetch("wiki", {"query": query})
 
     async def nsfw_scan(self, url: str):
         """
@@ -278,7 +318,7 @@ class ARQ:
 
                         results.data | results.data.drawings | results.data.hentai | .neutral | .sexy | .porn | .is_nsfw
         """
-        return await self._fetch("nsfw_scan", {"url": escape(url)})
+        return await self._fetch("nsfw_scan", {"url": url})
 
     async def stats(self):
         """
@@ -298,12 +338,12 @@ class ARQ:
         Returns An Object.
 
                 Parameters:
-                        min (min): Minimum limit
+                        min (int): Minimum limit
                         max (int): Maximum limit
                 Returns:
                         Result object (str): Result
         """
-        return await self._fetch("random", {"min": escape(min), "max": escape(max)})
+        return await self._fetch("random", {"min": min, "max": max})
 
     async def proxy(self):
         """
@@ -329,4 +369,62 @@ class ARQ:
 
                         results.id | .title | .overview | .rating | .releaseDate | .genre | .backdrop | .poster
         """
-        return await self._fetch("tmdb", {"query": escape(query)})
+        return await self._fetch("tmdb", {"query": query})
+
+    async def quotly(self, messages: [Message]):
+        """
+        Returns An Object.
+
+                Parameters:
+                        messages ([Message]): Generate quotly stickers.
+                Returns:
+                        Result object (str): Results which you can access with dot notation
+
+                        results
+        """
+        payload = {
+            "type": "quote",
+            "format": "png",
+            "backgroundColor": "#1b1429",
+            "messages": [
+                {
+                    "entities": [
+                        {
+                            "type": entity.type,
+                            "offset": entity.offset,
+                            "length": entity.length,
+                        }
+                        for entity in message.entities
+                    ]
+                    if message.entities
+                    else [],
+                    "chatId": message.from_user.id,
+                    "avatar": True,
+                    "from": {
+                        "id": message.from_user.id,
+                        "username": message.from_user.username
+                        if message.from_user.username
+                        else "",
+                        "photo": {
+                            "small_file_id": message.photo.small_file_id,
+                            "small_photo_unique_id": message.photo.small_photo_unique_id,
+                            "big_file_id": message.photo.big_file_id,
+                            "big_photo_unique_id": message.photo.big_photo_unique_id,
+                        }
+                        if message.photo
+                        else "",
+                        "type": message.chat.type,
+                        "name": (
+                            message.from_user.first_name + message.from_user.last_name
+                        )
+                        if message.from_user.last_name
+                        else message.from_user.first_name,
+                    },
+                    "text": message.text if message.text else "",
+                    "replyMessage": {},
+                }
+                for message in messages
+            ],
+        }
+
+        return await self._post("quotly", payload)
